@@ -5,7 +5,7 @@ imdbpy2sql.py script.
 This script puts the data of the plain text data files into a
 SQL database.
 
-Copyright 2005-2008 Davide Alberani <da@erlug.linux.it>
+Copyright 2005-2009 Davide Alberani <da@erlug.linux.it>
                2006 Giuseppe "Cowo" Corbelli <cowo --> lugbs.linux.it>
 
 This program is free software; you can redistribute it and/or modify
@@ -29,7 +29,8 @@ from gzip import GzipFile
 from types import UnicodeType
 from imdb.parser.sql.dbschema import *
 
-from imdb.parser.sql import soundex, get_movie_data
+from imdb.parser.common.locsql import soundex
+from imdb.parser.sql import get_movie_data
 from imdb.utils import analyze_title, analyze_name, \
         build_name, build_title, normalizeName, _articles, \
         build_company_name, analyze_company_name
@@ -119,7 +120,8 @@ MYSQLINNODB_OPTS = ['-e',
         '-e', 'END:FOR_EVERY_TABLE:ALTER TABLE %(table)s ENGINE=InnoDB;']
 SQLSERVER_OPTS = ['-e', 'BEFORE_EVERY_TODB:SET IDENTITY_INSERT %(table)s ON;',
         '-e', 'AFTER_EVERY_TODB:SET IDENTITY_INSERT %(table)s OFF;']
-SQLITE_OPTS = ['-e', 'BEFORE_EVERY_TODB:BEGIN TRANSACTION;',
+SQLITE_OPTS = ['-e', 'BEGIN:PRAGMA synchronous = OFF;',
+        '-e', 'BEFORE_EVERY_TODB:BEGIN TRANSACTION;',
         '-e', 'AFTER_EVERY_TODB:COMMIT;',
         '-e', 'BEFORE_INDEXES:BEGIN TRANSACTION;',
         'e', 'END:COMMIT;']
@@ -574,14 +576,42 @@ def _(s):
     s = s.encode(sys.stdout.encoding or 'utf_8', 'replace')
     return s
 
+if not hasattr(os, 'times'):
+    def times():
+        """Fake times() function."""
+        return (0.0, 0.0, 0.0, 0.0, 0.0)
+    os.times = times
+
 # Show time consumed by the single function call.
 CTIME = int(time.time())
 BEGIN_TIME = CTIME
-def t(s):
-    global CTIME
+CTIMES = os.times()
+BEGIN_TIMES = CTIMES
+
+def _minSec(*t):
+    """Return a tuple of (mins, secs, ...) - two for every item passed."""
+    l = []
+    for i in t:
+        l.extend(divmod(int(i), 60))
+    return tuple(l)
+
+def t(s, sinceBegin=False):
+    """Pretty-print timing information."""
+    global CTIME, CTIMES
     nt = int(time.time())
-    print '# TIME', s, ': %d min, %s sec.' % divmod(nt-CTIME, 60)
-    CTIME = nt
+    ntimes = os.times()
+    if not sinceBegin:
+        ct = CTIME
+        cts = CTIMES
+    else:
+        ct = BEGIN_TIME
+        cts = BEGIN_TIMES
+    print '# TIME', s, \
+            ': %dmin, %dsec (wall) %dmin, %dsec (user) %dmin, %dsec (system)' \
+            % _minSec(nt-ct, ntimes[0]-cts[0], ntimes[1]-cts[1])
+    if not sinceBegin:
+        CTIME = nt
+        CTIMES = ntimes
 
 def title_soundex(title):
     """Return the soundex code for the given title; the (optional) ending
@@ -754,7 +784,6 @@ def getSectionHash(fp):
         curTitle = ''
 
 NMMVSections = dict([(x, None) for x in ('MV: ', 'NM: ', 'OT: ', 'MOVI')])
-NMMVSectionsHASK = NMMVSections.has_key
 def getSectionNMMV(fp):
     """Return sections separated by lines starting with 'NM: ', 'MV: ',
     'OT: ' or 'MOVI'."""
@@ -763,7 +792,7 @@ def getSectionNMMV(fp):
     curNMMV = ''
     joiner = ''.join
     for line in fp:
-        if NMMVSectionsHASK(line[:4]):
+        if line[:4] in NMMVSections:
             if curSectList and curNMMV:
                 yield curNMMV, joiner(curSectList)
                 curSectList[:] = []
@@ -864,6 +893,7 @@ class _BaseCache(dict):
             except Exception, e:
                 if isinstance(e, KeyboardInterrupt):
                     raise
+                raise
                 print 'WARNING: unknown exception caught committing the data'
                 print 'WARNING: to the database; report this as a bug, since'
                 print 'WARNING: many data (%d items) were lost: %s' % \
@@ -898,7 +928,7 @@ class _BaseCache(dict):
     def addUnique(self, key, miscData=None):
         """Insert a new key and return its value; if the key is already
         in the dictionary, its previous  value is returned."""
-        if self.has_key(key): return self[key]
+        if key in self: return self[key]
         else: return self.add(key, miscData)
 
 
@@ -1180,6 +1210,49 @@ class CompaniesCache(_BaseCache):
             CSV_CURS.executemany(self.sqlstr, l)
 
 
+class KeywordsCache(_BaseCache):
+    """Manage the list of keywords."""
+    counter = counter()
+    className = 'KeywordsCache'
+
+    def __init__(self, *args, **kwds):
+        _BaseCache.__init__(self, *args, **kwds)
+        self._table_name = tableName(CompanyName)
+        self._id_for_custom_q = 'KEYWORDS'
+        self.flushEvery = 10000
+        self.sqlstr, self.converter = createSQLstr(Keyword, ['id', 'keyword',
+                                'phoneticCode'])
+
+    def populate(self):
+        print ' * POPULATING KeywordsCache...'
+        nameTbl = tableName(CompanyName)
+        keywordidCol = colName(Keyword, 'id')
+        keyCol = colName(Keyword, 'name')
+        CURS.execute('SELECT %s, %s FROM %s;' % (keywordidCol, keyCol,
+                                                    nameTbl))
+        _oldcacheValues = Keyword.sqlmeta.cacheValues
+        Keyword.sqlmeta.cacheValues = False
+        for x in fetchsome(CURS, self.flushEvery):
+            dict.__setitem__(self, x[1], x[0])
+        self.counter = counter(Keyword.select().count() + 1)
+        Keyword.sqlmeta.cacheValues = _oldcacheValues
+
+    def _toDB(self, quiet=0):
+        if not quiet:
+            print ' * FLUSHING KeywordsCache...'
+            sys.stdout.flush()
+        l = []
+        lapp = l.append
+        tmpDictiter = self._tmpDict.iteritems
+        for k, v in tmpDictiter():
+            keySoundex = soundex(k)
+            lapp((v, k, keySoundex))
+        if not CSV_DIR:
+            CURS.executemany(self.sqlstr, self.converter(l))
+        else:
+            CSV_CURS.executemany(self.sqlstr, l)
+
+
 class SQLData(dict):
     """Variable set of information, to be stored from time to time
     to the SQL database."""
@@ -1329,7 +1402,7 @@ def _parseColonList(lines, replaceKeys):
         k = cols[0]
         k = replaceKeys.get(k, k)
         v = ' '.join(cols[1:]).strip()
-        if not out.has_key(k): out[k] = []
+        if k not in out: out[k] = []
         out[k].append(v)
     return out
 
@@ -1533,7 +1606,7 @@ class AkasMoviesCache(MoviesCache):
             # id used to store this entry.
             the_id = item[0]
             # id of the referred title.
-            original_title_id = self.ids.get(the_id)
+            original_title_id = self.ids.get(the_id) or 0
             new_item = [the_id, original_title_id]
             new_item += item[1:-1]
             new_item.append(self.notes.get(the_id))
@@ -1920,14 +1993,14 @@ def doMovieCompaniesInfo():
         count = 0
         for line in fp:
             data = unpack(line.strip(), ('title', 'company', 'note'))
-            if not data.has_key('title'): continue
-            if not data.has_key('company'): continue
+            if 'title' not in data: continue
+            if 'company' not in data: continue
             title = data['title']
             company = data['company']
             mid = CACHE_MID.addUnique(title)
             cid = CACHE_COMPID.addUnique(company)
             note = None
-            if data.has_key('note'):
+            if 'note' in data:
                 note = data['note']
             if count % 10000 == 0:
                 print 'SCANNING %s:' % dataf[0][:-8].replace('-', ' '),
@@ -1942,8 +2015,6 @@ def doMovieCompaniesInfo():
 
 def doMiscMovieInfo():
     """Files with information on a single line about movies."""
-    sqldata = SQLData(table=MovieInfo,
-                cols=['movieID', 'infoTypeID', 'info', 'note'])
     for dataf in (('certificates.list.gz',CER_START),
                     ('color-info.list.gz',COL_START),
                     ('countries.list.gz',COU_START),
@@ -1963,6 +2034,12 @@ def doMiscMovieInfo():
         if typeindex == 'running times': typeindex = 'runtimes'
         elif typeindex == 'technical': typeindex = 'tech info'
         elif typeindex == 'language': typeindex = 'languages'
+        if typeindex != 'keywords':
+            sqldata = SQLData(table=MovieInfo,
+                        cols=['movieID', 'infoTypeID', 'info', 'note'])
+        else:
+            sqldata = SQLData(table=MovieKeyword,
+                        cols=['movieID', 'keywordID'])
         infoid =  INFO_TYPES[typeindex]
         count = 0
         if dataf[0] == 'locations.list.gz':
@@ -1971,19 +2048,27 @@ def doMiscMovieInfo():
             sqldata.flushEvery = 20000
         for line in fp:
             data = unpack(line.strip(), ('title', 'info', 'note'))
-            if not data.has_key('title'): continue
-            if not data.has_key('info'): continue
+            if 'title' not in data: continue
+            if 'info' not in data: continue
             title = data['title']
             mid = CACHE_MID.addUnique(title)
             note = None
-            if data.has_key('note'):
+            if 'note' in data:
                 note = data['note']
             if count % 10000 == 0:
                 print 'SCANNING %s:' % dataf[0][:-8].replace('-', ' '),
                 print _(data['title'])
-            sqldata.add((mid, infoid, data['info'], note))
+            info = data['info']
+            if typeindex == 'keywords':
+                keywordID = CACHE_KWRDID.addUnique(info)
+                sqldata.add((mid, keywordID))
+            else:
+                sqldata.add((mid, infoid, info, note))
             count += 1
         sqldata.flush()
+        if typeindex == 'keywords':
+            CACHE_KWRDID.flush()
+            CACHE_KWRDID.clear()
         fp.close()
         t('doMiscMovieInfo(%s)' % dataf[0][:-8].replace('-', ' '))
 
@@ -1992,13 +2077,13 @@ def getRating():
     """Movie's rating."""
     try: fp = SourceFile('ratings.list.gz', start=RAT_START, stop=RAT_STOP)
     except IOError: return
-    sqldata = SQLData(table=MovieInfo, cols=['movieID', 'infoTypeID',
-                                            'info', 'note'])
+    sqldata = SQLData(table=MovieInfoIdx, cols=['movieID', 'infoTypeID',
+                                                'info', 'note'])
     count = 0
     for line in fp:
         data = unpack(line, ('votes distribution', 'votes', 'rating', 'title'),
                         sep='  ')
-        if not data.has_key('title'): continue
+        if 'title' not in data: continue
         title = data['title'].strip()
         mid = CACHE_MID.addUnique(title)
         if count % 10000 == 0:
@@ -2019,7 +2104,7 @@ def getTopBottomRating():
         else: st = RAT_BOT10_START
         try: fp = SourceFile('ratings.list.gz', start=st, stop=TOPBOT_STOP)
         except IOError: break
-        sqldata = SQLData(table=MovieInfo,
+        sqldata = SQLData(table=MovieInfoIdx,
                     cols=['movieID',
                         RawValue('infoTypeID', INFO_TYPES[what]),
                         'info', 'note'])
@@ -2027,8 +2112,8 @@ def getTopBottomRating():
         print 'SCANNING %s...' % what
         for line in fp:
             data = unpack(line, ('votes distribution', 'votes', 'rank',
-                            'title'), sep='  ')
-            if not data.has_key('title'): continue
+                                'title'), sep='  ')
+            if 'title' not in data: continue
             title = data['title'].strip()
             mid = CACHE_MID.addUnique(title)
             if what == 'top 250 rank': rank = count
@@ -2094,6 +2179,7 @@ CACHE_PID = PersonsCache()
 CACHE_CID = CharactersCache()
 CACHE_CID.className = 'CharactersCache'
 CACHE_COMPID = CompaniesCache()
+CACHE_KWRDID = KeywordsCache()
 
 def _cmpfunc(x, y):
     """Sort a list of tuples, by the length of the first item (in reverse)."""
@@ -2381,8 +2467,15 @@ def run():
     CACHE_CID.clear()
     t('fushing caches...')
 
-    print 'TOTAL TIME TO INSERT/WRITE DATA: %d minutes, %d seconds' % \
-            divmod(int(time.time())-BEGIN_TIME, 60)
+    if CSV_DIR:
+        print 'loading CSV files into the database'
+        executeCustomQueries('BEFORE_CSV_LOAD')
+        loadCSVFiles()
+        t('loadCSVFiles()')
+
+    t('TOTAL TIME TO INSERT/WRITE DATA', sinceBegin=True)
+    #print 'TOTAL TIME TO INSERT/WRITE DATA: %d minutes, %d seconds' % \
+    #        divmod(int(time.time())-BEGIN_TIME, 60)
 
     executeCustomQueries('BEFORE_INDEXES')
 
@@ -2392,16 +2485,17 @@ def run():
     createIndexes(DB_TABLES)
     t('createIndexes()')
 
-    if CSV_DIR:
-        print 'loading CSV files into the database'
-        executeCustomQueries('BEFORE_CSV_LOAD')
-        loadCSVFiles()
-        t('loadCSVFiles()')
+    print 'adding foreign keys (this may take a while)'
+    sys.stdout.flush()
+    # Add FK.
+    createForeignKeys(DB_TABLES)
+    t('createForeignKeys()')
 
     executeCustomQueries('END')
 
-    print 'DONE! (in %d minutes, %d seconds)' % \
-            divmod(int(time.time())-BEGIN_TIME, 60)
+    t('FINAL', sinceBegin=True)
+    #print 'DONE! (in %d minutes, %d seconds)' % \
+    #        divmod(int(time.time())-BEGIN_TIME, 60)
 
 
 _HEARD = 0
@@ -2428,6 +2522,18 @@ def _kdb_handler(signum, frame):
 
 
 if __name__ == '__main__':
+    try:
+        print 'IMPORTING psyco...',
+        sys.stdout.flush()
+        #import DONOTIMPORTPSYCO
+        import psyco
+        #psyco.log()
+        psyco.profile()
+        print 'DONE!'
+        print ''
+    except ImportError:
+        print 'FAILED (not a big deal, everything is alright...)'
+        print ''
     import signal
     signal.signal(signal.SIGINT, _kdb_handler)
     run()
