@@ -25,13 +25,13 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 __all__ = ['IMDb', 'IMDbError', 'Movie', 'Person', 'Character', 'Company',
             'available_access_systems']
-__version__ = VERSION = '3.9'
+__version__ = VERSION = '4.0'
 
 # Import compatibility module (importing it is enough).
 import _compat
 
 import sys, os, ConfigParser
-from types import UnicodeType, TupleType, ListType, MethodType
+from types import MethodType
 
 from imdb import Movie, Person, Character, Company
 from imdb._exceptions import IMDbError, IMDbDataAccessError
@@ -56,6 +56,12 @@ imdbURL_character_main = imdbURL_character_base + 'ch%s/'
 imdbURL_company_base = '%scompany/' % imdbURL_base
 # http://akas.imdb.com/company/co%s/
 imdbURL_company_main = imdbURL_company_base + 'co%s/'
+# http://akas.imdb.com/keyword/%s/
+imdbURL_keyword_main = imdbURL_base + 'keyword/%s/'
+# http://akas.imdb.com/chart/top
+imdbURL_top250 = imdbURL_base + 'chart/top'
+# http://akas.imdb.com/chart/bottom
+imdbURL_bottom100 = imdbURL_base + 'chart/bottom'
 # http://akas.imdb.com/find?%s
 imdbURL_find = imdbURL_base + 'find?%s'
 
@@ -228,7 +234,7 @@ class IMDbBase:
     # in the subclasses).
     accessSystem = 'UNKNOWN'
 
-    def __init__(self, defaultModFunct=None, results=20,
+    def __init__(self, defaultModFunct=None, results=20, keywordsResults=100,
                 *arguments, **keywords):
         """Initialize the access system.
         If specified, defaultModFunct is the function used by
@@ -246,6 +252,13 @@ class IMDbBase:
         if results < 1:
             results = 20
         self._results = results
+        try:
+            keywordsResults = int(keywordsResults)
+        except (TypeError, ValueError):
+            keywordsResults = 100
+        if keywordsResults < 1:
+            keywordsResults = 100
+        self._keywordsResults = keywordsResults
 
     def _normalize_movieID(self, movieID):
         """Normalize the given movieID."""
@@ -354,7 +367,7 @@ class IMDbBase:
             results = 20
         # XXX: I suppose it will be much safer if the user provides
         #      an unicode string... this is just a guess.
-        if not isinstance(title, UnicodeType):
+        if not isinstance(title, unicode):
             title = unicode(title, encoding, 'replace')
         if not _episodes:
             res = self._search_movie(title, results)
@@ -414,7 +427,7 @@ class IMDbBase:
             results = int(results)
         except (ValueError, OverflowError):
             results = 20
-        if not isinstance(name, UnicodeType):
+        if not isinstance(name, unicode):
             name = unicode(name, encoding, 'replace')
         res = self._search_person(name, results)
         return [Person.Person(personID=self._get_real_personID(pi),
@@ -459,7 +472,7 @@ class IMDbBase:
             results = int(results)
         except (ValueError, OverflowError):
             results = 20
-        if not isinstance(name, UnicodeType):
+        if not isinstance(name, unicode):
             name = unicode(name, encoding, 'replace')
         res = self._search_character(name, results)
         return [Character.Character(characterID=self._get_real_characterID(pi),
@@ -504,22 +517,85 @@ class IMDbBase:
             results = int(results)
         except (ValueError, OverflowError):
             results = 20
-        if not isinstance(name, UnicodeType):
+        if not isinstance(name, unicode):
             name = unicode(name, encoding, 'replace')
         res = self._search_company(name, results)
         return [Company.Company(companyID=self._get_real_companyID(pi),
                 data=pd, modFunct=self._defModFunct,
                 accessSystem=self.accessSystem) for pi, pd in res][:results]
 
+    def _search_keyword(self, keyword, results):
+        """Return a list of 'keyword' strings."""
+        # XXX: for the real implementation, see the method of the
+        #      subclass, somewhere under the imdb.parser package.
+        raise NotImplementedError, 'override this method'
+
+    def search_keyword(self, keyword, results=None):
+        """Search for existing keywords, similar to the given one."""
+        if results is None:
+            results = self._keywordsResults
+        try:
+            results = int(results)
+        except (ValueError, OverflowError):
+            results = 100
+        if not isinstance(keyword, unicode):
+            keyword = unicode(keyword, encoding, 'replace')
+        return self._search_keyword(keyword, results)
+
+    def _get_keyword(self, keyword, results):
+        """Return a list of tuples (movieID, {movieData})"""
+        # XXX: for the real implementation, see the method of the
+        #      subclass, somewhere under the imdb.parser package.
+        raise NotImplementedError, 'override this method'
+
+    def get_keyword(self, keyword, results=None):
+        """Return a list of movies for the given keyword."""
+        if results is None:
+            results = self._keywordsResults
+        try:
+            results = int(results)
+        except (ValueError, OverflowError):
+            results = 100
+        # XXX: I suppose it will be much safer if the user provides
+        #      an unicode string... this is just a guess.
+        if not isinstance(keyword, unicode):
+            keyword = unicode(keyword, encoding, 'replace')
+        res = self._get_keyword(keyword, results)
+        return [Movie.Movie(movieID=self._get_real_movieID(mi),
+                data=md, modFunct=self._defModFunct,
+                accessSystem=self.accessSystem) for mi, md in res][:results]
+
+    def _get_top_bottom_movies(self, kind):
+        """Return the list of the top 250 or bottom 100 movies."""
+        # XXX: for the real implementation, see the method of the
+        #      subclass, somewhere under the imdb.parser package.
+        #      This method must return a list of (movieID, {movieDict})
+        #      tuples.  The kind parameter can be 'top' or 'bottom'.
+        raise NotImplementedError, 'override this method'
+
+    def get_top250_movies(self):
+        """Return the list of the top 250 movies."""
+        res = self._get_top_bottom_movies('top')
+        return [Movie.Movie(movieID=self._get_real_movieID(mi),
+                data=md, modFunct=self._defModFunct,
+                accessSystem=self.accessSystem) for mi, md in res]
+
+    def get_bottom100_movies(self):
+        """Return the list of the bottom 100 movies."""
+        res = self._get_top_bottom_movies('bottom')
+        return [Movie.Movie(movieID=self._get_real_movieID(mi),
+                data=md, modFunct=self._defModFunct,
+                accessSystem=self.accessSystem) for mi, md in res]
+
     def new_movie(self, *arguments, **keywords):
         """Return a Movie object."""
         # XXX: not really useful...
-        if keywords.has_key('title'):
-            if not isinstance(keywords['title'], UnicodeType):
+        if 'title' in keywords:
+            if not isinstance(keywords['title'], unicode):
                 keywords['title'] = unicode(keywords['title'],
                                             encoding, 'replace')
         elif len(arguments) > 1:
-            if not isinstance(arguments[1], UnicodeType):
+            if not isinstance(arguments[1], unicode):
                 arguments[1] = unicode(arguments[1], encoding, 'replace')
         return Movie.Movie(accessSystem=self.accessSystem,
                             *arguments, **keywords)
@@ -527,12 +603,12 @@ class IMDbBase:
     def new_person(self, *arguments, **keywords):
         """Return a Person object."""
         # XXX: not really useful...
-        if keywords.has_key('name'):
-            if not isinstance(keywords['name'], UnicodeType):
+        if 'name' in keywords:
+            if not isinstance(keywords['name'], unicode):
                 keywords['name'] = unicode(keywords['name'],
                                             encoding, 'replace')
         elif len(arguments) > 1:
-            if not isinstance(arguments[1], UnicodeType):
+            if not isinstance(arguments[1], unicode):
                 arguments[1] = unicode(arguments[1], encoding, 'replace')
         return Person.Person(accessSystem=self.accessSystem,
                                 *arguments, **keywords)
@@ -540,12 +616,12 @@ class IMDbBase:
     def new_character(self, *arguments, **keywords):
         """Return a Character object."""
         # XXX: not really useful...
-        if keywords.has_key('name'):
-            if not isinstance(keywords['name'], UnicodeType):
+        if 'name' in keywords:
+            if not isinstance(keywords['name'], unicode):
                 keywords['name'] = unicode(keywords['name'],
                                             encoding, 'replace')
         elif len(arguments) > 1:
-            if not isinstance(arguments[1], UnicodeType):
+            if not isinstance(arguments[1], unicode):
                 arguments[1] = unicode(arguments[1], encoding, 'replace')
         return Character.Character(accessSystem=self.accessSystem,
                                     *arguments, **keywords)
@@ -553,12 +629,12 @@ class IMDbBase:
     def new_company(self, *arguments, **keywords):
         """Return a Company object."""
         # XXX: not really useful...
-        if keywords.has_key('name'):
-            if not isinstance(keywords['name'], UnicodeType):
+        if 'name' in keywords:
+            if not isinstance(keywords['name'], unicode):
                 keywords['name'] = unicode(keywords['name'],
                                             encoding, 'replace')
         elif len(arguments) > 1:
-            if not isinstance(arguments[1], UnicodeType):
+            if not isinstance(arguments[1], unicode):
                 arguments[1] = unicode(arguments[1], encoding, 'replace')
         return Company.Company(accessSystem=self.accessSystem,
                                     *arguments, **keywords)
@@ -613,7 +689,7 @@ class IMDbBase:
                 info = self.get_character_infoset()
             else:
                 info = self.get_company_infoset()
-        if not isinstance(info, (TupleType, ListType)):
+        if not isinstance(info, (tuple, list)):
             info = (info,)
         res = {}
         for i in info:
@@ -624,18 +700,23 @@ class IMDbBase:
             except AttributeError:
                 raise IMDbDataAccessError, 'unknown information set "%s"' % i
             ret = method(mopID)
-            if ret.has_key('info sets'):
-                for ri in ret['info sets']:
-                    mop.add_to_current_info(ri)
-            else:
-                mop.add_to_current_info(i)
-            if ret.has_key('data'):
+            keys = None
+            if 'data' in ret:
                 res.update(ret['data'])
-            if ret.has_key('titlesRefs'):
+                if isinstance(ret['data'], dict):
+                    keys = ret['data'].keys()
+            if 'info sets' in ret:
+                for ri in ret['info sets']:
+                    mop.add_to_current_info(ri, keys, mainInfoset=i)
+                # So that only the 'main' required info set is set.
+                #mop.update_infoset_map(i, keys)
+            else:
+                mop.add_to_current_info(i, keys)
+            if 'titlesRefs' in ret:
                 mop.update_titlesRefs(ret['titlesRefs'])
-            if ret.has_key('namesRefs'):
+            if 'namesRefs' in ret:
                 mop.update_namesRefs(ret['namesRefs'])
-            if ret.has_key('charactersRefs'):
+            if 'charactersRefs' in ret:
                 mop.update_charactersRefs(ret['charactersRefs'])
         mop.set_data(res, override=0)
 
@@ -667,133 +748,64 @@ class IMDbBase:
         #      subclass, somewhere under the imdb.parser package.
         raise NotImplementedError, 'override this method'
 
-    def _searchIMDb(self, params):
-        """Fetch the given search page from the IMDb akas server."""
-        from imdb.parser.http import IMDbURLopener
-        url = imdbURL_find % params
-        content = u''
+    def _searchIMDb(self, kind, ton):
+        """Search the IMDb akas server for the given title or name."""
+        # The Exact Primary search system is gone AWL, so we resort
+        # to the mobile search. :-/
+        if not ton:
+            return None
+        aSystem = IMDb('mobile')
+        if kind == 'tt':
+            searchFunct = aSystem.search_movie
+            check = 'long imdb canonical title'
+        elif kind == 'nm':
+            searchFunct = aSystem.search_person
+            check = 'long imdb canonical name'
+        elif kind == 'char':
+            searchFunct = aSystem.search_character
+            check = 'long imdb canonical name'
+        elif kind == 'co':
+            # XXX: are [COUNTRY] codes included in the results?
+            searchFunct = aSystem.search_company
+            check = 'long imdb name'
         try:
-            urlOpener = IMDbURLopener()
-            content = urlOpener.retrieve_unicode(url)
-        except (IOError, IMDbDataAccessError):
-            pass
-        return content
+            searchRes = searchFunct(ton)
+        except IMDbError:
+            return None
+        # When only one result is returned, assume it was from an
+        # exact match.
+        if len(searchRes) == 1:
+            return searchRes[0].getID()
+        for item in searchRes:
+            # Return the first perfect match.
+            if item[check] == ton:
+                return item.getID()
+        return None
 
     def title2imdbID(self, title):
         """Translate a movie title (in the plain text data files format)
         to an imdbID.
         Try an Exact Primary Title search on IMDb;
         return None if it's unable to get the imdbID."""
-        if not title: return None
-        import urllib
-        if isinstance(title, unicode):
-            title = title.encode('utf-8')
-        params = 'q=%s;s=pt' % str(urllib.quote_plus(title))
-        content = self._searchIMDb(params)
-        if content and content[:512].find('<title>IMDb Title') != -1:
-            # Sometimes (e.g.: for titles with a "+" in it) a list
-            # of results is returned even for Exact Primary searches;
-            # this try to deal with it, hoping that a "normal" query
-            # will result in just on title.
-            params = 's=tt&q=%s' % str(urllib.quote_plus(title))
-            content = self._searchIMDb(params)
-        if not content: return None
-        from imdb.parser.http.searchMovieParser import DOMBasicMovieParser
-        mparser = DOMBasicMovieParser()
-        result = mparser.parse(content)
-        if not (result and result.get('data')): return None
-        return result['data'][0][0]
+        return self._searchIMDb('tt', title)
 
     def name2imdbID(self, name):
         """Translate a person name in an imdbID.
         Try an Exact Primary Name search on IMDb;
         return None if it's unable to get the imdbID."""
-        if not name: return None
-        import urllib
-        if isinstance(name, unicode):
-            name = name.encode('utf-8')
-        params = 'q=%s;s=pn' % str(urllib.quote_plus(name))
-        content = self._searchIMDb(params)
-        if content and content[:512].find('<title>IMDb Name') != -1:
-            params = 's=nm&q=%s' % str(urllib.quote_plus(name))
-            content = self._searchIMDb(params)
-        if not content: return None
-        from imdb.parser.http.searchPersonParser import DOMBasicPersonParser
-        pparser = DOMBasicPersonParser()
-        result = pparser.parse(content)
-        if not (result and result.get('data')): return None
-        return result['data'][0][0]
+        return self._searchIMDb('tt', name)
 
     def character2imdbID(self, name):
         """Translate a character name in an imdbID.
         Try an Exact Primary Name search on IMDb;
         return None if it's unable to get the imdbID."""
-        if not name: return None
-        import urllib
-        if isinstance(name, unicode):
-            name = name.encode('utf-8')
-        # XXX: s=pc is just a assumption of mine; it doesn't seem
-        #      to work, and I can't find a working alternative.
-        params = 'q=%s;s=pc' % str(urllib.quote_plus(name))
-        content = self._searchIMDb(params)
-        if not content: return None
-        if content[:512].find('<title>IMDb  Search') != -1:
-            from imdb.parser.http.searchCharacterParser import \
-                        DOMHTMLSearchCharacterParser
-            search_character_parser = DOMHTMLSearchCharacterParser()
-            result = search_character_parser.parse(content)
-            if not result: return None
-            if not result.has_key('data'): return None
-            if not result['data']: return None
-            # Try to read the first result.
-            chID, dname = result['data'][0]
-            if isinstance(dname.get('name'), unicode):
-                rname = dname['name'].encode('utf-8')
-            if name == rname:
-                return chID
-            return None
-        # XXX: still needed?
-        from imdb.parser.http.searchCharacterParser import \
-                    DOMBasicCharacterParser
-        cparser = DOMBasicCharacterParser()
-        result = cparser.parse(content)
-        if not (result and result.get('data')): return None
-        return result['data'][0][0]
+        return self._searchIMDb('char', name)
 
     def company2imdbID(self, name):
         """Translate a company name in an imdbID.
         Try an Exact Primary Name search on IMDb;
         return None if it's unable to get the imdbID."""
-        if not name: return None
-        import urllib
-        if isinstance(name, unicode):
-            name = name.encode('utf-8')
-        # XXX: s=pco is just a assumption of mine; it doesn't seem
-        #      to work, and I can't find a working alternative.
-        params = 'q=%s;s=pco' % str(urllib.quote_plus(name))
-        content = self._searchIMDb(params)
-        if not content: return None
-        if content[:512].find('<title>IMDb  Search') != -1:
-            from imdb.parser.http.searchCompanyParser import \
-                        DOMHTMLSearchCompanyParser
-            search_company_parser = DOMHTMLSearchCompanyParser()
-            result = search_company_parser.parse(content)
-            if not result: return None
-            if not result.has_key('data'): return None
-            if not result['data']: return None
-            # Try to read the first result.
-            chID, dname = result['data'][0]
-            if isinstance(dname.get('name'), unicode):
-                rname = dname['name'].encode('utf-8')
-            if name == rname:
-                return chID
-            return None
-        # Still needed?
-        from imdb.parser.http.searchCompanyParser import DOMBasicCompanyParser
-        cparser = DOMBasicCompanyParser()
-        result = cparser.parse(content)
-        if not (result and result.get('data')): return None
-        return result['data'][0][0]
+        return self._searchIMDb('co', name)
 
     def get_imdbID(self, mop):
         """Return the imdbID for the given Movie, Person, Character or Company
