@@ -7,7 +7,7 @@ the imdb.IMDb function will return an instance of this class when
 called with the 'accessSystem' argument set to "http" or "web"
 or "html" (this is the default).
 
-Copyright 2004-2008 Davide Alberani <da@erlug.linux.it>
+Copyright 2004-2009 Davide Alberani <da@erlug.linux.it>
                2008 H. Turgut Uyar <uyar@tekir.org>
 
 This program is free software; you can redistribute it and/or modify
@@ -31,7 +31,9 @@ from urllib import FancyURLopener, quote_plus
 from codecs import lookup
 
 from imdb import IMDbBase, imdbURL_movie_main, imdbURL_person_main, \
-                imdbURL_character_main, imdbURL_company_main, imdbURL_find
+                imdbURL_character_main, imdbURL_company_main, \
+                imdbURL_keyword_main, imdbURL_find, imdbURL_top250, \
+                imdbURL_bottom100
 from imdb.utils import analyze_title
 from imdb._exceptions import IMDbDataAccessError, IMDbParserError
 
@@ -39,10 +41,12 @@ import searchMovieParser
 import searchPersonParser
 import searchCharacterParser
 import searchCompanyParser
+import searchKeywordParser
 import movieParser
 import personParser
 import characterParser
 import companyParser
+import topBottomParser
 
 
 class _FakeParser(object):
@@ -60,9 +64,10 @@ class _ModuleProxy:
                 useModule=None, fallBackToNew=False):
         """Initialize a proxy for the given module; defaultKeys, if set,
         muste be a dictionary of values to set for instanced objects."""
-        self.oldParsers = oldParsers
+        if oldParsers or fallBackToNew:
+            warnings.warn('The old set of parsers was removed; falling ' \
+                    'back to the new parsers.')
         self.useModule = useModule
-        self.fallBackToNew = fallBackToNew
         if defaultKeys is None:
             defaultKeys = {}
         self._defaultKeys = defaultKeys
@@ -76,18 +81,9 @@ class _ModuleProxy:
             _entry = _sm._OBJECTS[name]
             # Initialize the parser.
             kwds = {}
-            if not self.oldParsers and self.useModule:
+            if self.useModule:
                 kwds = {'useModule': self.useModule}
-            parserClass = _entry[0][self.oldParsers]
-            # Warns when the old parser was requested, but it's gone.
-            if self.oldParsers and parserClass is _entry[0][0]:
-                if not self.fallBackToNew:
-                    warnings.warn('Old parser "%s" not available; these ' \
-                                'data will not be available.' % name)
-                    parserClass = _FakeParser
-                else:
-                    warnings.warn('Old parser "%s" not available; falling ' \
-                                'back to the new one.' % name)
+            parserClass = _entry[0][0]
             obj = parserClass(**kwds)
             attrsToSet = self._defaultKeys.copy()
             attrsToSet.update(_entry[1] or {})
@@ -276,6 +272,9 @@ class IMDbHTTPAccessSystem(IMDbBase):
         self.scompProxy = _ModuleProxy(searchCompanyParser, defaultKeys=_def,
                                     oldParsers=oldParsers, useModule=useModule,
                                     fallBackToNew=fallBackToNew)
+        self.skProxy = _ModuleProxy(searchKeywordParser, defaultKeys=_def,
+                                    oldParsers=oldParsers, useModule=useModule,
+                                    fallBackToNew=fallBackToNew)
         self.mProxy = _ModuleProxy(movieParser, defaultKeys=_def,
                                     oldParsers=oldParsers, useModule=useModule,
                                     fallBackToNew=fallBackToNew)
@@ -288,6 +287,10 @@ class IMDbHTTPAccessSystem(IMDbBase):
         self.compProxy = _ModuleProxy(companyParser, defaultKeys=_def,
                                     oldParsers=oldParsers, useModule=useModule,
                                     fallBackToNew=fallBackToNew)
+        self.topBottomProxy = _ModuleProxy(topBottomParser, defaultKeys=_def,
+                                    oldParsers=oldParsers, useModule=useModule,
+                                    fallBackToNew=fallBackToNew)
+
 
     def _normalize_movieID(self, movieID):
         """Normalize the given movieID."""
@@ -398,7 +401,7 @@ class IMDbHTTPAccessSystem(IMDbBase):
             params = params.replace('s=ep;', 's=tt;ttype=ep;', 1)
         cont = self._retrieve(imdbURL_find % params)
         #print 'URL:', imdbURL_find % params
-        if cont.find('more than 500 partial matches') == -1:
+        if cont.find('There were more than 500 partial matches') == -1:
             return cont
         # The retrieved page contains no results, because too many
         # titles or names contain the string we're looking for.
@@ -629,9 +632,9 @@ class IMDbHTTPAccessSystem(IMDbBase):
         cont = self._retrieve(imdbURL_person_main % personID + 'otherworks')
         return self.pProxy.otherworks_parser.parse(cont, getRefs=self._getRefs)
 
-    def get_person_agent(self, personID):
-        cont = self._retrieve(imdbURL_person_main % personID + 'agent')
-        return self.pProxy.agent_parser.parse(cont)
+    #def get_person_agent(self, personID):
+    #    cont = self._retrieve(imdbURL_person_main % personID + 'agent')
+    #    return self.pProxy.agent_parser.parse(cont)
 
     def get_person_publicity(self, personID):
         cont = self._retrieve(imdbURL_person_main % personID + 'publicity')
@@ -699,4 +702,36 @@ class IMDbHTTPAccessSystem(IMDbBase):
         ret = self.compProxy.company_main_parser.parse(cont)
         return ret
 
+    def _search_keyword(self, keyword, results):
+        # XXX: the IMDb web server seems to have some serious problem with
+        #      non-ascii keyword.
+        #      E.g.: http://akas.imdb.com/keyword/fianc%E9/
+        #      will return a 500 Internal Server Error: Redirect Recursion.
+        keyword = keyword.encode('utf8', 'ignore')
+        try:
+            cont = self._get_search_content('kw', keyword, results)
+        except IMDbDataAccessError:
+            return []
+        return self.skProxy.search_keyword_parser.parse(cont, results=results)['data']
+
+    def _get_keyword(self, keyword, results):
+        keyword = keyword.encode('utf8', 'ignore')
+        try:
+            cont = self._retrieve(imdbURL_keyword_main % keyword)
+        except IMDbDataAccessError:
+            return []
+        return self.skProxy.search_moviekeyword_parser.parse(cont, results=results)['data']
+
+    def _get_top_bottom_movies(self, kind):
+        if kind == 'top':
+            parser = self.topBottomProxy.top250_parser
+            url = imdbURL_top250
+        elif kind == 'bottom':
+            parser = self.topBottomProxy.bottom100_parser
+            url = imdbURL_bottom100
+        else:
+            return []
+        cont = self._retrieve(url)
+        return parser.parse(cont)['data']
+
 
