@@ -6,7 +6,7 @@ IMDb's data through a local installation.
 the imdb.IMDb function will return an instance of this class when
 called with the 'accessSystem' argument set to "local" or "files".
 
-Copyright 2004-2008 Davide Alberani <da@erlug.linux.it>
+Copyright 2004-2009 Davide Alberani <da@erlug.linux.it>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -30,8 +30,8 @@ from stat import ST_SIZE
 
 from imdb._exceptions import IMDbDataAccessError, IMDbError
 from imdb.utils import analyze_title, analyze_name, re_episodes, \
-                        normalizeName, analyze_company_name, \
-                        split_company_name_notes, normalizeTitle
+                        normalizeName, analyze_company_name, build_title, \
+                        split_company_name_notes, normalizeTitle, build_name
 
 from imdb.Movie import Movie
 from imdb.Company import Company
@@ -39,7 +39,8 @@ from personParser import getFilmography, getBio, getAkaNames
 from movieParser import getLabel, getMovieCast, getAkaTitles, parseMinusList, \
                         getPlot, getRatingData, getMovieMisc, getTaglines, \
                         getQuotes, getMovieLinks, getBusiness, getLiterature, \
-                        getLaserdisc, getMPAA
+                        getLaserdisc, getMPAA, searchSimilarKeywords, \
+                        getKeywordMovies, getTopBottomList
 from characterParser import getCharacterName, getCharacterFilmography
 from companyParser import getCompanyName, getCompanyFilmography, getCompanyID
 from utils import getFullIndex, KeyFScan, latin2utf
@@ -389,11 +390,11 @@ class IMDbLocalAccessSystem(IMDbLocalAndSqlAccessSystem):
             title1 = normalizeTitle(title)
             title2 = ''
             title3 = ''
-        resultsST = results
-        if not self.doAdult: resultsST = 0
+        # XXX: only a guess: results are shrinked, to exclude Adult
+        #      titles and to remove duplicated entries.
+        resultsST = results * 3
         res = _scan_titles('%stitles.key' % self.__db,
                             title1, title2, title3, resultsST, _episodes)
-        if self.doAdult and results > 0: res[:] = res[:results]
         res[:] = [x[1] for x in res]
         # Check for adult movies.
         if not self.doAdult:
@@ -406,8 +407,37 @@ class IMDbLocalAccessSystem(IMDbLocalAndSqlAccessSystem):
                                 attrKF='%sattributes.key' % self.__db)
                 if 'Adult' not in genres: newlist.append(entry)
             res[:] = newlist
-            if results > 0: res[:] = res[:results]
-        return res
+        # Get the real name, if this is an AKA.
+        # XXX: duplicated code!
+        new_res = []
+        seen_MID = []
+        for idx, (movieID, r) in enumerate(res):
+            # Remove duplicates.
+            # XXX: find a way to prefer titles with an AKA?  Or prefer
+            #      the original title?
+            if movieID in seen_MID:
+                continue
+            else:
+                seen_MID.append(movieID)
+            realMID = self._get_real_movieID(movieID)
+            if movieID == realMID:
+                new_res.append((movieID, r))
+                continue
+            if realMID in seen_MID:
+                continue
+            else:
+                seen_MID.append(realMID)
+            aka_title = build_title(r, canonical=1)
+            real_title = getLabel(realMID, '%stitles.index' % self.__db,
+                                '%stitles.key' % self.__db)
+            if aka_title == real_title:
+                new_res.append((realMID, r))
+                continue
+            new_r = analyze_title(real_title, canonical=1)
+            new_r['akas'] = [aka_title]
+            new_res.append((realMID, new_r))
+        if results > 0: new_res[:] = new_res[:results]
+        return new_res
 
     def _search_episode(self, title, results):
         title = title.strip()
@@ -602,6 +632,8 @@ class IMDbLocalAccessSystem(IMDbLocalAndSqlAccessSystem):
         mq = getQuotes(movieID, '%squotes.data' % self.__db,
                             '%squotes.index' % self.__db)
         trefs, nrefs = self._extractRefs(mq)
+        for idx, quote in enumerate(mq):
+            mq[idx] = quote.split('::')
         if mq: return {'data': {'quotes': mq},
                         'titlesRefs': trefs, 'namesRefs': nrefs}
         return {'data': {}}
@@ -734,9 +766,36 @@ class IMDbLocalAccessSystem(IMDbLocalAndSqlAccessSystem):
         name1, name2, name3 = nameVariations(name)
         res =  _scan_names('%snames.key' % self.__db,
                             name1, name2, name3, results)
-        ##if results > 0: res[:] = res[:results]
         res[:] = [x[1] for x in res]
-        return res
+        new_res = []
+        seen_PID = []
+        for idx, (personID, r) in enumerate(res):
+            # Remove duplicates.
+            # XXX: find a way to prefer names with an AKA?  Or prefer
+            #      the original name?
+            if personID in seen_PID:
+                continue
+            else:
+                seen_PID.append(personID)
+            realPID = self._get_real_personID(personID)
+            if personID == realPID:
+                new_res.append((personID, r))
+                continue
+            if realPID in seen_PID:
+                continue
+            else:
+                seen_PID.append(realPID)
+            aka_name = build_name(r, canonical=1)
+            real_name = getLabel(realPID, '%snames.index' % self.__db,
+                                '%snames.key' % self.__db)
+            if aka_name == real_name:
+                new_res.append((realPID, r))
+                continue
+            new_r = analyze_name(real_name, canonical=1)
+            new_r['akas'] = [aka_name]
+            new_res.append((realPID, new_r))
+        if results > 0: new_res[:] = new_res[:results]
+        return new_res
 
     def get_person_main(self, personID):
         infosets = ('main', 'biography', 'other works')
@@ -907,3 +966,36 @@ class IMDbLocalAccessSystem(IMDbLocalAndSqlAccessSystem):
             res.update(filmography)
         return {'data': res}
 
+    def _search_keyword(self, keyword, results):
+        return searchSimilarKeywords(keyword,
+                            '%skeywords.data' % self.__db)[:results]
+
+    def _get_keyword(self, keyword, results):
+        return [(movieID, analyze_title(getLabel(movieID,
+                '%stitles.index' % self.__db, '%stitles.key' % self.__db)))
+                for movieID in getKeywordMovies(keyword,
+                    '%skeywords.data' % self.__db)][:results]
+
+    def _get_top_bottom_movies(self, kind):
+        if kind == 'top':
+            kind = 'top 250 rank'
+        elif kind == 'bottom':
+            kind = 'bottom 10 rank'
+        else:
+            return []
+        info = getTopBottomList(kind, '%stopbottom.db' % self.__db)
+        if not info:
+            return []
+        res = []
+        for d in info:
+            if not 'movieID' in d:
+                continue
+            movieID = d['movieID']
+            del d['movieID']
+            minfo = analyze_title(getLabel(movieID,
+                                    '%stitles.index' % self.__db,
+                                    '%stitles.key' % self.__db))
+            minfo.update(d)
+            res.append((movieID, minfo))
+        return res
+
