@@ -6,7 +6,7 @@ IMDb's data for mobile systems.
 the imdb.IMDb function will return an instance of this class when
 called with the 'accessSystem' argument set to "mobile".
 
-Copyright 2005-2008 Davide Alberani <da@erlug.linux.it>
+Copyright 2005-2009 Davide Alberani <da@erlug.linux.it>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -25,7 +25,6 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 import re
 from urllib import unquote
-from types import ListType, TupleType
 
 from imdb import imdbURL_movie_main, imdbURL_person_main, imdbURL_character_main
 from imdb.Movie import Movie
@@ -89,14 +88,15 @@ def _getTagsWith(s, cont, toClosure=False, maxRes=None):
     return lres
 
 
-def _findBetween(s, begins, ends, beginindx=0, maxRes=None):
+def _findBetween(s, begins, ends, beginindx=0, maxRes=None, lres=None):
     """Return the list of strings from the 's' string which are included
     between the 'begins' and 'ends' strings."""
-    lres = []
+    if lres is None:
+        lres = []
     bi = s.find(begins, beginindx)
     if bi != -1:
         lbegins = len(begins)
-        if isinstance(ends, (ListType, TupleType)):
+        if isinstance(ends, (list, tuple)):
             eset = [s.find(end, bi+lbegins) for end in ends]
             eset[:] = [x for x in eset if x != -1]
             if not eset: ei = -1
@@ -107,13 +107,11 @@ def _findBetween(s, begins, ends, beginindx=0, maxRes=None):
             match = s[bi+lbegins:ei]
             lres.append(match)
             if maxRes is not None and len(lres) >= maxRes: return lres
-            lres += _findBetween(s, begins, ends, ei)
+            _findBetween(s, begins, ends, beginindx=ei, maxRes=maxRes,
+                        lres=lres)
     return lres
 
 
-# Remove AKAs.
-_reAKAS = re.compile(r'aka <em.*?</td>', re.I | re.M)
-
 class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
     """The class used to access IMDb's data through the web for
     mobile terminals."""
@@ -183,7 +181,7 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
         if not title: return res
         tl = title[0].lower()
         if not tl.startswith('imdb title'):
-            # XXX: a direct hit!
+            # a direct hit!
             title = _unHtml(title[0])
             midtag = _getTagsWith(cont, 'name="arg"', maxRes=1)
             if not midtag: midtag = _getTagsWith(cont, 'name="auto"', maxRes=1)
@@ -197,14 +195,33 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
                 title += ' (mini)'
             res[:] = [(str(mid[0]), analyze_title(title, canonical=1))]
         else:
-            cont = _reAKAS.sub('</td>', cont)
-            lis = _findBetween(cont, 'td valign="top">', ['</td>', '</small>'])
+            # XXX: this results*3 prevents some recursion errors, but...
+            #      it's not exactly understandable (i.e.: why 'results' is
+            #      not enough to get all the results?)
+            lis = _findBetween(cont, 'td valign="top">', '</td>',
+                                maxRes=results*3)
             for li in lis:
+                akaIdx = li.find('aka <em>')
+                akas = []
+                if akaIdx != -1:
+                    akas = [_unHtml(x) for x in li[akaIdx:].split('<br>')]
+                    li = li[:akaIdx]
+                if akas:
+                    for idx, aka in enumerate(akas):
+                        aka = aka.replace('" - ', '::')
+                        if aka.startswith('aka "'):
+                            aka = aka[5:]
+                        if aka[-1] == '"':
+                            aka = aka[:-1]
+                        akas[idx] = aka
                 imdbid = re_imdbID.findall(li)
                 mtitle = _unHtml(li)
                 if not (imdbid and mtitle): continue
                 mtitle = mtitle.replace('(TV mini-series)', '(mini)')
-                res.append((str(imdbid[0]), analyze_title(mtitle, canonical=1)))
+                resd = analyze_title(mtitle, canonical=1)
+                if akas:
+                    resd['akas'] = akas
+                res.append((str(imdbid[0]), resd))
         return res
 
     def get_movie_main(self, movieID):
@@ -455,7 +472,7 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
         if not name: return res
         nl = name[0].lower()
         if not nl.startswith('imdb name'):
-            # XXX: a direct hit!
+            # a direct hit!
             name = _unHtml(name[0])
             # Easiest way: the board link (for person who already have
             # messages in the board).
@@ -475,13 +492,21 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
             if not (pid and name): return res
             res[:] = [(str(pid[0]), analyze_name(name, canonical=1))]
         else:
-            lis = _findBetween(cont, 'td valign="top">',
-                                ['<small', '</td>', '<br> aka'])
+            lis = _findBetween(cont, 'td valign="top">', '</td>',
+                                maxRes=results*3)
             for li in lis:
+                akas = _findBetween(li, '<em>"', '"</em>')
+                for sep in ['<small', '<br> aka', '<br> birth name']:
+                    sepIdx = li.find(sep)
+                    if sepIdx != -1:
+                        li = li[:sepIdx]
                 pid = re_imdbID.findall(li)
                 pname = _unHtml(li)
                 if not (pid and pname): continue
-                res.append((str(pid[0]), analyze_name(pname, canonical=1)))
+                resd = analyze_name(pname, canonical=1)
+                if akas:
+                    resd['akas'] = akas
+                res.append((str(pid[0]), resd))
         return res
 
     def get_person_main(self, personID, _parseChr=False):
@@ -694,7 +719,7 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
         nl = name[0].lower()
         if not (nl.startswith('imdb search') or nl.startswith('imdb  search') \
                 or nl.startswith('imdb character')):
-            # XXX: a direct hit!
+            # a direct hit!
             name = _unHtml(name[0]).replace('(Character)', '').strip()
             pidtag = _getTagsWith(cont, '/character/ch', maxRes=1)
             pid = None
@@ -703,8 +728,10 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
             if not (pid and name): return res
             res[:] = [(str(pid[0]), analyze_name(name, canonical=0))]
         else:
-            sects = _findBetween(cont, '<b>Popular Characters</b>', '</table>')
-            sects += _findBetween(cont, '<b>Characters', '</table>')
+            sects = _findBetween(cont, '<b>Popular Characters</b>', '</table>',
+                                maxRes=results*3)
+            sects += _findBetween(cont, '<b>Characters', '</table>',
+                                maxRes=results*3)
             for sect in sects:
                 lis = _findBetween(sect, '<a href="/character/',
                                     ['<small', '</td>', '<br'])
@@ -729,7 +756,7 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
             if intro:
                 d['introduction'] = intro
         bios = _findBetween(cont, '<div class="display">',
-                            '<div class="history">', maxRes=1)
+                            '<div class="history">')
         if bios:
             bios = _findBetween(bios[0], '<h4>', ('<h4>', '</div>'))
         if bios:
@@ -740,6 +767,7 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
                 bio = bio.replace('<br/>', '\n')
                 bio = subSGMLRefs(re_unhtmlsub('', bio).strip())
                 bio = bio.replace(' ::', '::').replace(':: ', '::')
+                bio = bio.replace('::', ': ', 1)
                 if bio:
                     d.setdefault('biography', []).append(bio)
         return {'data': d}
