@@ -3,11 +3,11 @@ parser.sql package (imdb package).
 
 This package provides the IMDbSqlAccessSystem class used to access
 IMDb's data through a SQL database.  Every database supported by
-the SQLObject Object Relational Manager is available.
+the SQLObject _AND_ SQLAlchemy Object Relational Managers is available.
 the imdb.IMDb function will return an instance of this class when
 called with the 'accessSystem' argument set to "sql", "database" or "db".
 
-Copyright 2005-2008 Davide Alberani <da@erlug.linux.it>
+Copyright 2005-2009 Davide Alberani <da@erlug.linux.it>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -27,12 +27,12 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 # FIXME: this whole module was written in a veeery short amount of time.
 #        The code should be commented, rewritten and cleaned. :-)
 
-from types import UnicodeType
 import warnings
 
 from imdb.parser.common.locsql import IMDbLocalAndSqlAccessSystem, \
                     scan_names, scan_titles, titleVariations, \
-                    nameVariations, merge_roles, scan_company_names
+                    nameVariations, merge_roles, scan_company_names, \
+                    soundex, filterSimilarKeywords
 from imdb.utils import normalizeTitle, normalizeName, build_title, \
                         build_name, analyze_name, analyze_title, \
                         build_company_name, re_episodes, _articles
@@ -41,32 +41,6 @@ from imdb.Movie import Movie
 from imdb.Company import Company
 from imdb._exceptions import IMDbDataAccessError, IMDbError
 
-try:
-    from imdb.parser.common.cutils import soundex
-except ImportError:
-    warnings.warn('Unable to import the cutils.soundex function.'
-                    '  Searches of movie titles and person names will be'
-                    ' a bit slower.')
-
-    _translate = dict(B='1', C='2', D='3', F='1', G='2', J='2', K='2', L='4',
-                      M='5', N='5', P='1', Q='2', R='6', S='2', T='3', V='1',
-                      X='2', Z='2')
-    _translateget = _translate.get
-
-    def soundex(s):
-        """Return the soundex code for the given string."""
-        # Maximum length of the soundex code.
-        SOUNDEX_LEN = 5
-        if not s: return None
-        s = s.upper()
-        soundCode =  s[0]
-        for c in s[1:]:
-            cw = _translateget(c, '0')
-            if cw != '0' and soundCode[-1] != cw:
-                soundCode += cw
-        return soundCode[:SOUNDEX_LEN]
-
-
 _litlist = ['screenplay/teleplay', 'novel', 'adaption', 'book',
             'production process protocol', 'interviews',
             'printed media reviews', 'essays', 'other literature']
@@ -152,6 +126,48 @@ def get_movie_data(movieID, kindDict, fromAka=0):
     return mdict
 
 
+def _iterKeywords(results):
+    """Iterate over (key.id, key.keyword) columns of a selection of
+    the Keyword table."""
+    for key in results:
+        yield key.id, key.keyword
+
+
+def getSingleInfo(table, movieID, infoType, notAList=False):
+    """Return a dictionary in the form {infoType: infoListOrString},
+    retrieving a single set of information about a given movie, from
+    the specified table."""
+    infoTypeID = InfoType.select(InfoType.q.info == infoType)
+    if infoTypeID.count() == 0:
+        return {}
+    res = table.select(AND(table.q.movieID == movieID,
+                        table.q.infoTypeID == infoTypeID[0].id))
+    retList = []
+    for r in res:
+        info = r.info
+        note = r.note
+        if note:
+            info += u'::%s' % note
+        retList.append(info)
+    if not retList:
+        return {}
+    if not notAList: return {infoType: retList}
+    else: return {infoType: retList[0]}
+
+
+def _cmpTop(a, b, what='top 250 rank'):
+    """Compare function used to sort top 250/bottom 10 rank."""
+    av = int(a[1].get(what))
+    bv = int(b[1].get(what))
+    if av == bv:
+        return 0
+    return (-1, 1)[av > bv]
+
+def _cmpBottom(a, b):
+    """Compare function used to sort top 250/bottom 10 rank."""
+    return _cmpTop(a, b, what='bottom 10 rank')
+
+
 class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
     """The class used to access IMDb's data through a SQL database."""
 
@@ -177,11 +193,11 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
                 if mod == 'sqlalchemy':
                     from alchemyadapter import getDBTables, NotFoundError, \
                                                 setConnection, AND, OR, IN, \
-                                                ISNULL, toUTF8
+                                                ISNULL, CONTAINSSTRING, toUTF8
                 elif mod == 'sqlobject':
                     from objectadapter import getDBTables, NotFoundError, \
                                                 setConnection, AND, OR, IN, \
-                                                ISNULL, toUTF8
+                                                ISNULL, CONTAINSSTRING, toUTF8
                 else:
                     warnings.warn('unknown module "%s".' % mod)
                     continue
@@ -190,7 +206,8 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
                 #      through the whole module.
                 for k, v in [('NotFoundError', NotFoundError),
                             ('AND', AND), ('OR', OR), ('IN', IN),
-                            ('ISNULL', ISNULL)]:
+                            ('ISNULL', ISNULL),
+                            ('CONTAINSSTRING', CONTAINSSTRING)]:
                     globals()[k] = v
                 self.toUTF8 = toUTF8
                 DB_TABLES = getDBTables(uri)
@@ -450,7 +467,7 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
         else:
             _episodes = False
             s_title = normalizeTitle(s_title)
-        if isinstance(s_title, UnicodeType):
+        if isinstance(s_title, unicode):
             s_title = s_title.encode('ascii', 'ignore')
 
         soundexCode = soundex(s_title)
@@ -512,13 +529,11 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
             raise IMDbDataAccessError, \
                     'unable to search the database: "%s"' % str(e)
 
-        resultsST = results
-        if not self.doAdult: resultsST = 0
+        resultsST = results * 3
         res = scan_titles(qr, title1, title2, title3, resultsST,
                             searchingEpisode=episodeOf is not None,
                             onlyEpisodes=_episodes,
                             ro_thresold=0.0)
-        if self.doAdult and results > 0: res[:] = res[:results]
         res[:] = [x[1] for x in res]
 
         if res and not self.doAdult:
@@ -530,8 +545,24 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
                                 MovieInfo.q.info == 'Adult',
                                 IN(MovieInfo.q.movieID, mids)))]
             res[:] = [x for x in res if x[0] not in adultlist]
-            if results > 0: res[:] = res[:results]
-        return res
+
+        new_res = []
+        # XXX: can there be duplicated?
+        for r in res:
+            if r not in q2:
+                new_res.append(r)
+                continue
+            mdict = r[1]
+            aka_title = build_title(mdict, canonical=1, ptdf=1)
+            orig_dict = get_movie_data(r[0], self._kind)
+            orig_title = build_title(orig_dict, canonical=1, ptdf=1)
+            if aka_title == orig_title:
+                new_res.append(r)
+                continue
+            orig_dict['akas'] = [aka_title]
+            new_res.append((r[0], orig_dict))
+        if results > 0: new_res[:] = new_res[:results]
+        return new_res
 
     def _search_episode(self, title, results):
         return self._search_movie(title, results, _episodes=True)
@@ -580,6 +611,10 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
         # Info about the movie.
         minfo = [(self._info[m.infoTypeID], m.info, m.note)
                 for m in MovieInfo.select(MovieInfo.q.movieID == movieID)]
+        minfo += [(self._info[m.infoTypeID], m.info, m.note)
+                for m in MovieInfoIdx.select(MovieInfoIdx.q.movieID == movieID)]
+        minfo += [('keywords', Keyword.get(m.keywordID).keyword, None)
+                for m in MovieKeyword.select(MovieKeyword.q.movieID == movieID)]
         minfo = _groupListBy(minfo, 0)
         for group in minfo:
             sect = group[0][0]
@@ -675,6 +710,9 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
         #        else: nl.append(i)
         #    res['plot'][:] = nl
         # Other transformations.
+        if 'quotes' in res:
+            for idx, quote in enumerate(res['quotes']):
+                res['quotes'][idx] = quote.split('::')
         if res.has_key('runtimes') and len(res['runtimes']) > 0:
             rt = res['runtimes'][0]
             episodes = re_episodes.findall(rt)
@@ -695,6 +733,12 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
         if res.has_key('guest'):
             res['guests'] = res['guest']
             del res['guest']
+        if 'top 250 rank' in res:
+            try: res['top 250 rank'] = int(res['top 250 rank'])
+            except: pass
+        if 'bottom 10 rank' in res:
+            try: res['bottom 10 rank'] = int(res['bottom 10 rank'])
+            except: pass
         trefs,nrefs = {}, {}
         trefs,nrefs = self._extractRefs(sub_dict(res,Movie.keys_tomodify_list))
         return {'data': res, 'titlesRefs': trefs, 'namesRefs': nrefs,
@@ -727,7 +771,7 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
         if not name: return []
         s_name = analyze_name(name)['name']
         if not s_name: return []
-        if isinstance(s_name, UnicodeType):
+        if isinstance(s_name, unicode):
             s_name = s_name.encode('ascii', 'ignore')
         soundexCode = soundex(s_name)
         name1, name2, name3 = nameVariations(name)
@@ -748,8 +792,10 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
         try:
             qr = [(q.id, {'name': q.name, 'imdbIndex': q.imdbIndex})
                     for q in Name.select(condition)]
-            qr += [(q.personID, {'name': q.name, 'imdbIndex': q.imdbIndex})
+            
+            q2 = [(q.personID, {'name': q.name, 'imdbIndex': q.imdbIndex})
                     for q in AkaName.select(conditionAka)]
+            qr += q2
         except NotFoundError, e:
             raise IMDbDataAccessError, \
                     'unable to search the database: "%s"' % str(e)
@@ -763,7 +809,28 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
             if tmpd['imdbIndex'] is None:
                 del tmpd['imdbIndex']
             returnl.append((x[0], tmpd))
-        return returnl
+
+        new_res = []
+        # XXX: can there be duplicated?
+        for r in returnl:
+            if r not in q2:
+                new_res.append(r)
+                continue
+            pdict = r[1]
+            aka_name = build_name(pdict, canonical=1)
+            p = Name.get(r[0])
+            orig_dict = {'name': p.name, 'imdbIndex': p.imdbIndex}
+            if orig_dict['imdbIndex'] is None:
+                del orig_dict['imdbIndex']
+            orig_name = build_name(orig_dict, canonical=1)
+            if aka_name == orig_name:
+                new_res.append(r)
+                continue
+            orig_dict['akas'] = [aka_name]
+            new_res.append((r[0], orig_dict))
+        if results > 0: new_res[:] = new_res[:results]
+
+        return new_res
 
     def get_person_main(self, personID):
         # Every person information is retrieved from here.
@@ -887,7 +954,7 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
         if not name: return []
         s_name = analyze_name(name)['name']
         if not s_name: return []
-        if isinstance(s_name, UnicodeType):
+        if isinstance(s_name, unicode):
             s_name = s_name.encode('ascii', 'ignore')
         s_name = normalizeName(s_name)
         soundexCode = soundex(s_name)
@@ -980,7 +1047,7 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
     def _search_company(self, name, results):
         name = name.strip()
         if not name: return []
-        if isinstance(name, UnicodeType):
+        if isinstance(name, unicode):
             name = name.encode('ascii', 'ignore')
         soundexCode = soundex(name)
         # If the soundex is None, compare only with the first
@@ -1041,6 +1108,53 @@ class IMDbSqlAccessSystem(IMDbLocalAndSqlAccessSystem):
             res.get(ctype, []).sort()
         return {'data': res, 'info sets': infosets}
 
+    def _search_keyword(self, keyword, results):
+        constr = OR(Keyword.q.phoneticCode ==
+                    soundex(keyword.encode('ascii', 'ignore')),
+                    CONTAINSSTRING(Keyword.q.keyword, self.toUTF8(keyword)))
+        return filterSimilarKeywords(keyword,
+                        _iterKeywords(Keyword.select(constr)))[:results]
+
+    def _get_keyword(self, keyword, results):
+        keyID = Keyword.select(Keyword.q.keyword == keyword)
+        if keyID.count() == 0:
+            return []
+        keyID = keyID[0].id
+        movies = MovieKeyword.select(MovieKeyword.q.keywordID ==
+                                    keyID)[:results]
+        return [(m.movieID, get_movie_data(m.movieID, self._kind))
+                for m in movies]
+
+    def _get_top_bottom_movies(self, kind):
+        if kind == 'top':
+            kind = 'top 250 rank'
+        elif kind == 'bottom':
+            # Not a refuse: the plain text data files contains only
+            # the bottom 10 movies.
+            kind = 'bottom 10 rank'
+        else:
+            return []
+        infoID = InfoType.select(InfoType.q.info == kind)
+        if infoID.count() == 0:
+            return []
+        infoID = infoID[0].id
+        movies = MovieInfoIdx.select(MovieInfoIdx.q.infoTypeID == infoID)
+        ml = []
+        for m in movies:
+            minfo = get_movie_data(m.movieID, self._kind)
+            for k in kind, 'votes', 'rating', 'votes distribution':
+                valueDict = getSingleInfo(MovieInfoIdx, m.movieID,
+                                            k, notAList=True)
+                if k in (kind, 'votes') and k in valueDict:
+                    valueDict[k] = int(valueDict[k])
+                elif k == 'rating' and k in valueDict:
+                    valueDict[k] = float(valueDict[k])
+                minfo.update(valueDict)
+            ml.append((m.movieID, minfo))
+        sorter = (_cmpBottom, _cmpTop)[kind == 'top 250 rank']
+        ml.sort(sorter)
+        return ml
+
     def __del__(self):
         """Ensure that the connection is closed."""
         if not hasattr(self, '_connection'): return
