@@ -3,7 +3,8 @@ utils module (imdb package).
 
 This module provides basic utilities for the imdb package.
 
-Copyright 2004-2008 Davide Alberani <da@erlug.linux.it>
+Copyright 2004-2009 Davide Alberani <da@erlug.linux.it>
+               2009 H. Turgut Uyar <uyar@tekir.org>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -22,7 +23,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 from __future__ import generators
 import re
-from types import UnicodeType, StringType, ListType, TupleType, DictType
+import string
 from copy import copy, deepcopy
 from time import strptime, strftime
 
@@ -184,7 +185,7 @@ for article in _articles:
 def canonicalTitle(title):
     """Return the title in the canonic format 'Movie Title, The'."""
     try:
-        if _articlesDict.has_key(title.split(', ')[-1].lower()): return title
+        if title.split(', ')[-1].lower() in _articlesDict: return title
     except IndexError: pass
     ltitle = title.lower()
     for article in _spArticles:
@@ -209,7 +210,7 @@ def canonicalTitle(title):
 def normalizeTitle(title):
     """Return the title in the normal "The Title" format."""
     stitle = title.split(', ')
-    if len(stitle) > 1 and _articlesDict.has_key(stitle[-1].lower()):
+    if len(stitle) > 1 and stitle[-1].lower() in _articlesDict:
         sep = ' '
         if stitle[-1][-1] in ("'", '-'): sep = ''
         title = '%s%s%s' % (stitle[-1], sep, ', '.join(stitle[:-1]))
@@ -685,20 +686,21 @@ def modifyStrings(o, modFunct, titlesRefs, namesRefs, charactersRefs):
     """Modify a string (or string values in a dictionary or strings
     in a list), using the provided modFunct function and titlesRefs
     namesRefs and charactersRefs references dictionaries."""
-    if isinstance(o, (UnicodeType, StringType)):
+    # Notice that it doesn't go any deeper than the first two levels in a list.
+    if isinstance(o, (unicode, str)):
         return modFunct(o, titlesRefs, namesRefs, charactersRefs)
-    elif isinstance(o, (ListType, TupleType)):
+    elif isinstance(o, (list, tuple, dict)):
         _stillorig = 1
-        if isinstance(o, ListType): keys = xrange(len(o))
+        if isinstance(o, (list, tuple)): keys = xrange(len(o))
         else: keys = o.keys()
         for i in keys:
             v = o[i]
-            if isinstance(v, (UnicodeType, StringType)):
+            if isinstance(v, (unicode, str)):
                 if _stillorig:
                     o = copy(o)
                     _stillorig = 0
                 o[i] = modFunct(v, titlesRefs, namesRefs, charactersRefs)
-            elif isinstance(v, (ListType, TupleType)):
+            elif isinstance(v, (list, tuple)):
                 modifyStrings(o[i], modFunct, titlesRefs, namesRefs,
                             charactersRefs)
     return o
@@ -739,8 +741,240 @@ class RolesList(list):
         return u' / '.join([unicode(x).encode('utf8') for x in self])
 
 
+# Replace & with &amp;, but only if it's not already part of a charref.
+_re_amp = re.compile(r'(&)(?!\w+;)', re.I)
+
+def escape4xml(value):
+    """Escape some chars that can't be present in a XML value."""
+    if isinstance(value, int):
+        value = str(value)
+    value = _re_amp.sub('&amp;', value)
+    value = value.replace('"', '&quot;').replace("'", '&apos;')
+    value = value.replace('<', '&lt;').replace('>', '&gt;')
+    if isinstance(value, unicode):
+        value = value.encode('ascii', 'xmlcharrefreplace')
+    return value
+
+
+def _refsToReplace(value, modFunct, titlesRefs, namesRefs, charactersRefs):
+    """Return three lists - for movie titles, persons and characters names -
+    with two items tuples: the first item is the reference once escaped
+    by the user-provided modFunct function, the second is the same
+    reference un-escaped."""
+    mRefs = []
+    for refRe, refTemplate in [(re_titleRef, u'_%s_ (qv)'),
+                                (re_nameRef, u"'%s' (qv)"),
+                                (re_characterRef, u'#%s# (qv)')]:
+        theseRefs = []
+        for theRef in refRe.findall(value):
+            # refTemplate % theRef values don't change for a single
+            # _Container instance, so this is a good candidate for a
+            # cache or something - even if it's so rarely used that...
+            # Moreover, it can grow - ia.update(...) - and change if
+            # modFunct is modified.
+            goodValue = modFunct(refTemplate % theRef, titlesRefs, namesRefs,
+                                charactersRefs)
+            toReplace = escape4xml(goodValue)
+            # Only the 'value' portion is replaced.
+            replaceWith = goodValue.replace(theRef, escape4xml(theRef))
+            theseRefs.append((toReplace, replaceWith))
+        mRefs.append(theseRefs)
+    return mRefs
+
+
+def _handleTextNotes(s):
+    """Split text::notes strings."""
+    ssplit = s.split('::', 1)
+    if len(ssplit) == 1:
+        return s
+    return u'%s<notes>%s</notes>' % (ssplit[0], ssplit[1])
+
+
+def _normalizeValue(value, withRefs=False, modFunct=None, titlesRefs=None,
+                    namesRefs=None, charactersRefs=None):
+    """Replace some chars that can't be present in a XML text."""
+    # XXX: use s.encode(encoding, 'xmlcharrefreplace') ?  Probably not
+    #      a great idea: after all, returning a unicode is safe.
+    if isinstance(value, (unicode, str)):
+        if not withRefs:
+            value = _handleTextNotes(escape4xml(value))
+        else:
+            # Replace references that were accidentally escaped.
+            replaceLists = _refsToReplace(value, modFunct, titlesRefs,
+                                        namesRefs, charactersRefs)
+            value = modFunct(value, titlesRefs or {}, namesRefs or {},
+                            charactersRefs or {})
+            value = _handleTextNotes(escape4xml(value))
+            for replaceList in replaceLists:
+                for toReplace, replaceWith in replaceList:
+                    value = value.replace(toReplace, replaceWith)
+    else:
+        value = unicode(value)
+    return value
+
+
+def _tag4TON(ton, addAccessSystem=False):
+    """Build a tag for the given _Container instance;
+    both open and close tags are returned."""
+    tag = ton.__class__.__name__.lower()
+    what = 'name'
+    if tag == 'movie':
+        value = ton['long imdb title']
+        what = 'title'
+    else:
+        value = ton['long imdb name']
+    value = _normalizeValue(value)
+    extras = u''
+    crl = ton.currentRole
+    if crl:
+        if not isinstance(crl, list):
+            crl = [crl]
+        for cr in crl:
+            crTag = cr.__class__.__name__.lower()
+            crValue = cr['long imdb name']
+            crValue = _normalizeValue(crValue)
+            crID = cr.getID()
+            if crID is not None:
+                extras += u'<current-role><%s id="%s">' \
+                            u'<name>%s</name></%s>' % (crTag, crID,
+                                                        crValue, crTag)
+            else:
+                extras += u'<current-role><%s><name>%s</name></%s>' % \
+                               (crTag, crValue, crTag)
+            if cr.notes:
+                extras += u'<notes>%s</notes>' % _normalizeValue(cr.notes)
+            extras += u'</current-role>'
+    theID = ton.getID()
+    if theID is not None:
+        beginTag = u'<%s id="%s"' % (tag, theID)
+        if addAccessSystem and ton.accessSystem:
+            beginTag += ' access-system="%s"' % ton.accessSystem
+        beginTag += u'><%s>%s</%s>' % (what, value, what)
+    else:
+        beginTag = u'<%s><%s>%s</%s>' % (tag, what, value, what)
+    beginTag += extras
+    if ton.notes:
+        beginTag += u'<notes>%s</notes>' % _normalizeValue(ton.notes)
+    return (beginTag, u'</%s>' % tag)
+
+
+TAGS_TO_MODIFY = {
+    'movie.parents-guide': ('item', 'title'),
+    'movie.number-of-votes': ('item', 'title'),
+    'movie.soundtrack.item': ('item', 'title'),
+    'movie.quotes': ('quote', None),
+    'movie.quotes.quote': ('line', None),
+    'movie.demographic': ('item', 'title'),
+    'movie.episodes': ('season', 'number'),
+    'movie.episodes.season': ('episode', 'number'),
+    'person.merchandising-links':  ('item', 'title'),
+    'person.genres':  ('item', 'title'),
+    'person.quotes':  ('quote', None),
+    'person.keywords':  ('item', 'title'),
+    'character.quotes': ('item', 'title'),
+    'character.quotes.item': ('quote', None),
+    'character.quotes.item.quote': ('line', None)
+    }
+
+_allchars = string.maketrans('', '')
+_keepchars = _allchars.translate(_allchars, string.ascii_lowercase + '-' +
+                                 string.digits)
+
+def _tagAttr(key, fullpath):
+    """Return a tuple with a tag name and a (possibly empty) attribute,
+    applying the conversions specified in TAGS_TO_MODIFY and checking
+    that the tag is safe for a XML document."""
+    value_attr = None
+    attrs = u''
+    if fullpath in TAGS_TO_MODIFY:
+        tagName, value_attr = TAGS_TO_MODIFY[fullpath]
+    elif not isinstance(key, unicode):
+        if isinstance(key, str):
+            tagName = unicode(key, 'ascii', 'ignore')
+        else:
+            tagName = unicode(key)
+    else:
+        tagName = key
+    tagName = tagName.lower().replace(' ', '-')
+    origTagName = tagName
+    tagName = str(tagName).translate(_allchars, _keepchars)
+    if (not tagName) or tagName[0].isdigit() or tagName[0] == '-' or \
+            origTagName != tagName:
+        # This is a fail-safe: we should never be here, since unpredictable
+        # keys must be listed in TAGS_TO_MODIFY.
+        # This will break the DTD/schema, but at least it will produce a
+        # valid XML.
+        #print 'ERROR - INVALID TAG: %s [%s]' % (escape4xml(key), fullpath)
+        tagName = 'item'
+        value_attr = 'key'
+    if value_attr is not None:
+        attrs = u'%s="%s"' % (value_attr, escape4xml(key))
+    return tagName, attrs
+
+
+def _seq2xml(seq, _l=None, withRefs=False, modFunct=None,
+            titlesRefs=None, namesRefs=None, charactersRefs=None,
+            _topLevel=True, key2infoset=None, fullpath=''):
+    """Convert a sequence or a dictionary to a list of XML
+    unicode strings."""
+    if _l is None:
+        _l = []
+    if isinstance(seq, dict):
+        for key in seq:
+            if isinstance(key, _Container):
+                # Here we're assuming that a _Container is never a top-level
+                # key (otherwise we should handle key2infoset).
+                openTag, closeTag = _tag4TON(key)
+                # So that fullpath will contains something meaningful.
+                tagName = key.__class__.__name__.lower()
+            else:
+                tagName, attrs = _tagAttr(key, fullpath)
+                openTag = u'<%s' % tagName
+                if attrs:
+                    openTag += ' %s' % attrs
+                if _topLevel and key2infoset and key in key2infoset:
+                    openTag += u' infoset="%s"' % key2infoset[key]
+                openTag += u'>'
+                closeTag = u'</%s>' % tagName
+            _l.append(openTag)
+            _seq2xml(seq[key], _l, withRefs, modFunct, titlesRefs,
+                    namesRefs, charactersRefs, _topLevel=False,
+                    fullpath='%s.%s' % (fullpath, tagName))
+            _l.append(closeTag)
+    elif isinstance(seq, (list, tuple)):
+        tagName, attrs = _tagAttr('item', fullpath)
+        beginTag = u'<%s' % tagName
+        if attrs:
+            beginTag += u' %s' % attrs
+        beginTag += u'>'
+        closeTag = u'</%s>' % tagName
+        for item in seq:
+            if isinstance(item, _Container):
+                _seq2xml(item, _l, withRefs, modFunct, titlesRefs,
+                         namesRefs, charactersRefs, _topLevel=False,
+                         fullpath='%s.%s' % (fullpath,
+                                    item.__class__.__name__.lower()))
+            else:
+                _l.append(beginTag)
+                _seq2xml(item, _l, withRefs, modFunct, titlesRefs,
+                        namesRefs, charactersRefs, _topLevel=False,
+                        fullpath='%s.%s' % (fullpath, tagName))
+                _l.append(closeTag)
+    else:
+        if isinstance(seq, _Container):
+            _l.extend(_tag4TON(seq))
+        else:
+            # Text, ints, floats and the like.
+            _l.append(_normalizeValue(seq, withRefs=withRefs,
+                                        modFunct=modFunct,
+                                        titlesRefs=titlesRefs,
+                                        namesRefs=namesRefs,
+                                        charactersRefs=charactersRefs))
+    return _l
+
+
 class _Container(object):
-    """Base class for Movie, Person and Character classes."""
+    """Base class for Movie, Person, Character and Company classes."""
      # The default sets of information retrieved.
     default_info = ()
 
@@ -750,13 +984,14 @@ class _Container(object):
     # List of keys to modify.
     keys_tomodify_list = ()
 
+    # Function used to compare two instances of this class.
     cmpFunct = None
 
     def __init__(self, myID=None, data=None, notes=u'',
                 currentRole=u'', roleID=None, roleIsPerson=False,
                 accessSystem=None, titlesRefs=None, namesRefs=None,
                 charactersRefs=None, modFunct=None, *args, **kwds):
-        """Initialize a Movie, Person or Character object.
+        """Initialize a Movie, Person, Character or Company object.
         *myID* -- your personal identifier for this object.
         *data* -- a dictionary used to initialize the object.
         *notes* -- notes for the person referred in the currentRole
@@ -844,7 +1079,7 @@ class _Container(object):
 
     def _set_currentRole(self, role):
         """Set self.currentRole to a Character or Person instance."""
-        if isinstance(role, (UnicodeType, StringType)):
+        if isinstance(role, (unicode, str)):
             if not role:
                 self.__role = None
             else:
@@ -853,7 +1088,7 @@ class _Container(object):
         elif isinstance(role, (list, tuple)):
             self.__role = RolesList()
             for item in role:
-                if isinstance(item, (UnicodeType, StringType)):
+                if isinstance(item, (unicode, str)):
                     self.__role.append(self._roleClass(name=item,
                                         accessSystem=self.accessSystem,
                                         modFunct=self.modFunct))
@@ -880,6 +1115,8 @@ class _Container(object):
         self.charactersRefs = {}
         self.modFunct = modClearRefs
         self.current_info = []
+        self.infoset2keys = {}
+        self.key2infoset = {}
         self.__role = None
         self._reset()
 
@@ -893,6 +1130,8 @@ class _Container(object):
         self.namesRefs = {}
         self.charactersRefs = {}
         self.current_info = []
+        self.infoset2keys = {}
+        self.key2infoset = {}
         self.__role = None
         self._clear()
 
@@ -902,14 +1141,28 @@ class _Container(object):
         """Return the current set of information retrieved."""
         return self.current_info
 
+    def update_infoset_map(self, infoset, keys, mainInfoset):
+        """Update the mappings between infoset and keys."""
+        if keys is None:
+            keys = []
+        if mainInfoset is not None:
+            theIS = mainInfoset
+        else:
+            theIS = infoset
+        self.infoset2keys[theIS] = keys
+        for key in keys:
+            self.key2infoset[key] = theIS
+
     def set_current_info(self, ci):
         """Set the current set of information retrieved."""
+        # XXX:Remove? It's never used and there's no way to update infoset2keys.
         self.current_info = ci
 
-    def add_to_current_info(self, val):
+    def add_to_current_info(self, val, keys=None, mainInfoset=None):
         """Add a set of information to the current list."""
         if val not in self.current_info:
             self.current_info.append(val)
+            self.update_infoset_map(val, keys, mainInfoset)
 
     def has_current_info(self, val):
         """Return true if the given set of information is in the list."""
@@ -990,6 +1243,44 @@ class _Container(object):
         """Number of items in the data dictionary."""
         return len(self.data)
 
+    def getAsXML(self, key):
+        """Return a XML representation of the specified key, or None
+        if empty."""
+        # Prevent modifyStrings in __getitem__ to be called; if needed,
+        # it will be called by the _normalizeValue function.
+        origModFunct = self.modFunct
+        self.modFunct = modNull
+        try:
+            withRefs = False
+            if key in self.keys_tomodify and \
+                    origModFunct not in (None, modNull):
+                withRefs = True
+            value = self.get(key)
+            if value is None:
+                return None
+            tag = self.__class__.__name__.lower()
+            return u''.join(_seq2xml({key: value}, withRefs=withRefs,
+                                        modFunct=origModFunct,
+                                        titlesRefs=self.titlesRefs,
+                                        namesRefs=self.namesRefs,
+                                        charactersRefs=self.charactersRefs,
+                                        key2infoset=self.key2infoset,
+                                        fullpath=tag))
+        finally:
+            self.modFunct = origModFunct
+
+    def asXML(self):
+        """Return a XML representation of the whole object."""
+        beginTag, endTag = _tag4TON(self, addAccessSystem=True)
+        resList = [beginTag]
+        for key in self.keys():
+            value = self.getAsXML(key)
+            if not value:
+                continue
+            resList.append(value)
+        resList.append(endTag)
+        return u''.join(resList)
+
     def _getitem(self, key):
         """Handle special keys."""
         return None
@@ -1003,7 +1294,7 @@ class _Container(object):
         # Handle key aliases.
         key = self.keys_alias.get(key, key)
         rawData = self.data[key]
-        if self.keys_tomodify.has_key(key) and \
+        if key in self.keys_tomodify and \
                 self.modFunct not in (None, modNull):
             try:
                 return modifyStrings(rawData, self.modFunct, self.titlesRefs,
@@ -1111,7 +1402,7 @@ class _Container(object):
         return deepcopy(self)
 
 
-def flatten(seq, toDescend=(ListType, DictType, TupleType),
+def flatten(seq, toDescend=(list, dict, tuple),
             yieldDictKeys=0, onlyKeysType=(_Container,), scalar=None):
     """Iterate over nested lists and dictionaries; toDescend is a list
     or a tuple of types to be considered non-scalar; if yieldDictKeys is
@@ -1121,7 +1412,7 @@ def flatten(seq, toDescend=(ListType, DictType, TupleType),
         if scalar is None or isinstance(seq, scalar):
             yield seq
     else:
-        if isinstance(seq, (DictType, _Container)):
+        if isinstance(seq, (dict, _Container)):
             if yieldDictKeys:
                 # Yield also the keys of the dictionary.
                 for key in seq.iterkeys():
